@@ -1,19 +1,23 @@
 """Parse a PTL template, return AST node.
 
-First template function names are mangled, noting the template type.
+First h-strings (F-strings that are to become htmltext) are marked.
 Next, the file is parsed into a parse tree.  This tree is converted into
 a modified AST.  It is during this state that the semantics are modified
 by adding extra nodes to the tree.  The modified AST is returned, ready
 to pass to the compile() built-in function.
 """
 
-import re
+import sys
 import io
 import ast
 import tokenize
 
-# special marker for h-strings, cannot appear in source code literals
-HSTRING_MARKER = 'HSTRING\xa0\x00'
+# A special marker for h-strings, added as a prefix by token
+# translator. It is not allowed to appear in source code literals.
+# This is not so elegant but seems the simplist way to pass along the
+# information from the tokenizer to the AST that a string should be
+# htmltext.
+HSTRING_MARKER = '\x02HSTRING\x03'
 
 TEMPLATE_TYPES = {
     'ptl_html': 'html',
@@ -21,104 +25,115 @@ TEMPLATE_TYPES = {
 }
 
 
-def translate_defs(tokens):
-    # Rename the function name for html/plain templates.  The special names
-    # will be recognized by the AST transformer.
-    NAME = tokenize.NAME
-    OP = tokenize.OP
+def _is_h_str_tok(tok):
+    """Return True if the token is a h-string (f-string as htmltext)."""
+    prefix = tok.string[:2]
+    return prefix in {'F"', "F'"}
+
+
+def _translate_hstrings_lexical(tokens):
+    """Find h-string literals and annotate them so that the AST transform
+    can turn them into htmltext values.
+    """
+    # This version handles Python >= 3.12, post PEP 701.
+
+    FSTRING_START = tokenize.FSTRING_START
+    FSTRING_MIDDLE = tokenize.FSTRING_MIDDLE
+    FSTRING_END = tokenize.FSTRING_END
+
+    # A stack to track if we are currently inside of a h-string.  Since
+    # f-strings can be nested, this needs to be a stack and not just a
+    # boolean.
+    in_h_string = [False]
     i = 0
     while i < len(tokens):
         tok = tokens[i]
-        if tok.type == NAME and tok.string == 'def':
-            if tokens[i + 2][:2] == (OP, '[') and tokens[i + 4][:2] == (
-                OP,
-                ']',
-            ):
-                name_tok = list(tokens[i + 1])
-                prefix = '_q_%s_template_' % tokens[i + 3].string
-                name_tok[1] = prefix + name_tok[1]
-                del tokens[i + 2 : i + 5]
-                tokens[i + 1] = tokenize.TokenInfo(*name_tok)
+        if tok.type == FSTRING_START:
+            in_h_string.append(_is_h_str_tok(tokens[i]))
+        elif tok.type == FSTRING_END:
+            in_h_string.pop()
+        if (
+            tok.type == FSTRING_START
+            and in_h_string[-1]
+            and tokens[i + 1].type != FSTRING_MIDDLE
+        ):
+            # Ensure there is always a FSTRING_MIDDLE token following the
+            # FSTRING_START token.  In Python >= 3.12+, an empty f-string
+            # has no FSTRING_MIDDLE token.  Also, in the case of nested
+            # f-strings, there might be no middle token. Since the code below
+            # requires a middle value in order to mark the string as an
+            # h-string, insert a middle token with an empty value and the
+            # h-string marker.
+            new_tok = tokenize.TokenInfo(
+                FSTRING_MIDDLE, '', tok.end, tok.end, tok.line
+            )
+            tokens.insert(i + 1, new_tok)
+            assert tokens[i].type == FSTRING_START
+            assert tokens[i + 1].type == FSTRING_MIDDLE
+        elif tok.type == FSTRING_MIDDLE and in_h_string[-1]:
+            # Handle Python 3.12+ F-strings.
+            if _has_str_marker(tok):
+                raise SyntaxError(
+                    'invalid str literal, cannot contain h-string marker'
+                )
+            # prefix string value with marker
+            tok_str = HSTRING_MARKER + tok.string
+            new_tok = tokenize.TokenInfo(
+                tok.type, tok_str, tok.start, tok.end, tok.line
+            )
+            # replace FSTRING_MIDDLE token with new token with prefixed value
+            tokens[i] = new_tok
         i += 1
 
 
-def translate_hstrings(tokens):
-    # Detect h-string literals.  They should up in the token stream
-    # as two tokens, e.g. h"foo":
-    #   NAME    'h'
-    #   STRING  '"foo"'
-    #
-    # We translate them to:
-    #
-    #   STRING f'"<HSTRING_PREFIX>foo"'
-    #
-    # The AST transformer will detect the prefix and wrap them in
-    # calls of htmltext().
-    STRING = tokenize.STRING
-    NAME = tokenize.NAME
+def _translate_hstrings_tokenized(tokens):
+    """Find h-string literals and annotate them so that the AST transform
+    can turn them into htmltext values.
+    """
+    # This version handles Python < 3.12, pre PEP 701.
     i = 0
     while i < len(tokens):
         tok = tokens[i]
-        if tok.type == STRING:
+        if tok.type == tokenize.STRING and _is_h_str_tok(tok):
+            # For Python < 3.12, h-strings show up as a STRING token.  The
+            # value of the token must start with F" or F'
             if i == 0:
                 i += 1
                 continue
-            need_dedent = False
-            prefix = tok.string[:2]
-            if prefix in {'F"', "F'"}:
-                # found a F-prefixed string
-                have_h_string = True
-                str_tok = list(tok)
-                s = tok[1][1:]  # string value with 'F' stripped
-            elif tokens[i - 1].type == NAME and tokens[i - 1].string == 'h':
-                # found a h-prefixed string
-                have_h_string = True
-                str_tok = list(tokens[i])
-                s = str_tok[1]
-                # h prefix is separate token, remove it
-                del tokens[i - 1]
-                i -= 1
-                need_dedent = True
-            else:
-                have_h_string = False
-            if have_h_string:
-                s = ast.literal_eval(s)
-                if HSTRING_MARKER in s:
-                    raise SyntaxError(
-                        'invalid str literal, cannot contain '
-                        'h-string marker, got %r' % s
-                    )
-                # prefix string with marker.  Putting the marker inside the
-                # string is not so elegant but it is an easy way to pass
-                # the annotation along to the AST transformer.  The check
-                # above ensures that no real string contains the marker.
-                s = HSTRING_MARKER + s
-                s = repr(s)
-                if '{' in s:
-                    # h-strings do f-string like evaluation, if the string
-                    # doesn't contain any f-string braces, just use a normal
-                    # string as that generates more efficient bytecode
-                    s = 'f' + s
-                str_tok[1] = s
-                if need_dedent:
-                    # deindent one character, we stripped (NAME 'h') token
-                    srow, scol = str_tok[2]
-                    str_tok[2] = (srow, scol - 1)
-                tokens[i] = tokenize.TokenInfo(*str_tok)
+            # found a F-prefixed string
+            str_tok = list(tok)
+            s = tok[1][1:]  # string value with 'F' stripped
+            s = ast.literal_eval(s)
+            if _has_str_marker(tok):
+                raise SyntaxError(
+                    'invalid str literal, cannot contain '
+                    'h-string marker, got %r' % s
+                )
+            # prefix the string value with the marker
+            s = HSTRING_MARKER + s
+            s = repr(s)
+            if '{' in s:
+                # h-strings do f-string like evaluation, if the string
+                # doesn't contain any f-string braces, just use a normal
+                # string as that generates more efficient bytecode
+                s = 'f' + s
+            str_tok[1] = s
+            tokens[i] = tokenize.TokenInfo(*str_tok)
         i += 1
 
 
+if sys.hexversion > 0x30C0000:
+    translate_hstrings = _translate_hstrings_lexical
+else:
+    translate_hstrings = _translate_hstrings_tokenized
+
+
 def translate_source(buf, filename='<string>'):
     """
     Since we can't modify the parser in the builtin parser module we
     must do token translation here.  Luckily it does not affect line
     numbers.
 
-    def foo [plain] (...): -> def _q_plain_template__foo(...):
-
-    def foo [html] (...): -> def _q_html_template__foo(...):
-
-    h'foo' -> f'<HSTRING_PREFIX>foo'
     F'foo' -> f'<HSTRING_PREFIX>foo'
     """
     assert isinstance(buf, bytes)
@@ -129,17 +144,47 @@ def translate_source(buf, filename='<string>'):
         exc_type = type(exc)
         raise exc_type(str(exc), (filename, exc.lineno, exc.offset, exc.text))
     translate_hstrings(tokens)
-    translate_defs(tokens)
     ut = tokenize.Untokenizer()
     src = ut.untokenize(tokens)
     return ut.encoding, src
 
 
-def _is_str_node(n):
-    # Python 3.8 replaces Str with Constant
-    return isinstance(n, ast.Str) or (
-        isinstance(n, ast.Constant) and isinstance(n.value, str)
-    )
+if sys.hexversion > 0x30C0000:
+
+    def _is_str_node(n):
+        return isinstance(n, ast.Constant) and isinstance(n.value, str)
+
+    def _has_str_marker(n):
+        return _is_str_node(n) and HSTRING_MARKER in n.value
+
+    def _h_str_replace(n):
+        return ast.Constant(n.value.replace(HSTRING_MARKER, ''))
+
+    def _ast_const(value):
+        return ast.Constant(value)
+
+    def _ast_num(value):
+        return ast.Constant(value)
+
+else:
+
+    def _is_str_node(n):
+        # Python 3.8 replaces Str with Constant
+        return isinstance(n, ast.Str) or (
+            isinstance(n, ast.Constant) and isinstance(n.value, str)
+        )
+
+    def _has_str_marker(n):
+        return _is_str_node(n) and HSTRING_MARKER in n.s
+
+    def _h_str_replace(n):
+        return ast.Str(n.s.replace(HSTRING_MARKER, ''))
+
+    def _ast_const(value):
+        return ast.NameConstant(value)
+
+    def _ast_num(value):
+        return ast.Num(value)
 
 
 class TemplateTransformer(ast.NodeTransformer):
@@ -191,17 +236,11 @@ class TemplateTransformer(ast.NodeTransformer):
     def visit_FunctionDef(self, node):
         name = node.name
         template_type = None
-        # look for @html or @plain decorator
+        # look for @ptl_html or @ptl_plain decorator
         for dec in node.decorator_list:
             if isinstance(dec, ast.Name):
                 if dec.id in TEMPLATE_TYPES:
                     template_type = TEMPLATE_TYPES[dec.id]
-        if template_type is None:
-            # look for old-style [html] or [plain] annotation
-            m = re.match('_q_(html|plain)_template_(.*)', name)
-            if m:
-                template_type = m.group(1)
-                name = m.group(2)
         if template_type is None:
             self.__template_type.append(None)
             node = self.generic_visit(node)
@@ -212,14 +251,8 @@ class TemplateTransformer(ast.NodeTransformer):
 
             # _q_output = _q_TemplateIO(template_type == 'html')
             klass = ast.Name(id='_q_TemplateIO', ctx=ast.Load())
-            arg = ast.NameConstant(template_type == 'html')
-            instance = ast.Call(
-                func=klass,
-                args=[arg],
-                keywords=[],
-                starargs=None,
-                kwargs=None,
-            )
+            arg = _ast_const(template_type == 'html')
+            instance = ast.Call(func=klass, args=[arg], keywords=[])
             assign_name = ast.Name(id='_q_output', ctx=ast.Store())
             assign = ast.Assign(targets=[assign_name], value=instance)
             ast.copy_location(assign, node)
@@ -238,9 +271,7 @@ class TemplateTransformer(ast.NodeTransformer):
             # return _q_output.getvalue()
             n = ast.Name(id='_q_output', ctx=ast.Load())
             n = ast.Attribute(value=n, attr='getvalue', ctx=ast.Load())
-            n = ast.Call(
-                func=n, args=[], keywords=[], starargs=None, kwargs=None
-            )
+            n = ast.Call(func=n, args=[], keywords=[])
             ret = ast.Return(value=n)
             ast.copy_location(ret, node.body[-1])
             ast.fix_missing_locations(ret)
@@ -256,13 +287,7 @@ class TemplateTransformer(ast.NodeTransformer):
             # stack, call _q_output(obj).
             name = ast.Name(id='_q_output', ctx=ast.Load())
             ast.copy_location(name, node)
-            call = ast.Call(
-                func=name,
-                args=[node.value],
-                keywords=[],
-                starargs=None,
-                kwargs=None,
-            )
+            call = ast.Call(func=name, args=[node.value], keywords=[])
             ast.copy_location(call, node)
             expr = ast.Expr(call)
             return ast.copy_location(expr, node)
@@ -271,18 +296,16 @@ class TemplateTransformer(ast.NodeTransformer):
     def visit_Constant(self, node, html=False):
         if not _is_str_node(node):
             return node
-        if html or HSTRING_MARKER in node.s:
+        if html or _has_str_marker(node):
             # found h-string marker, remove it.  Note that marker can appear
             # within the string if there is a string continued over two lines
             # using backslash.
-            s = ast.Str(node.s.replace(HSTRING_MARKER, ''))
+            s = _h_str_replace(node)
             ast.copy_location(s, node)
             # wrap in call to _q_htmltext
             n = ast.Name(id='_q_htmltext', ctx=ast.Load())
             ast.copy_location(n, node)
-            n = ast.Call(
-                func=n, args=[s], keywords=[], starargs=None, kwargs=None
-            )
+            n = ast.Call(func=n, args=[s], keywords=[])
             return ast.copy_location(n, node)
         return node
 
@@ -295,7 +318,7 @@ class TemplateTransformer(ast.NodeTransformer):
         # In CPython, it is done with the BUILD_STRING opcode.  For
         # h-strings, we call quixote.html._q_join() instead.
         for v in node.values:
-            if _is_str_node(v) and HSTRING_MARKER in v.s:
+            if _has_str_marker(v):
                 break  # need to use _q_join()
         else:
             # none of the join arguments are htmltext, just use normal
@@ -307,14 +330,17 @@ class TemplateTransformer(ast.NodeTransformer):
             if isinstance(v, ast.FormattedValue):
                 v = self.visit_FormattedValue(v, html=True)
             elif _is_str_node(v):
+                if v.value == HSTRING_MARKER:
+                    # The translate_hstrings() function can insert empty
+                    # strings into the values list.  Discard them here as
+                    # a small performance optimization.
+                    continue
                 v = self.visit_Constant(v, html=True)
             else:
                 v = self.generic_visit(v)
             values.append(v)
         n = ast.Name(id='_q_join', ctx=ast.Load())
-        n = ast.Call(
-            func=n, args=values, keywords=[], starargs=None, kwargs=None
-        )
+        n = ast.Call(func=n, args=values, keywords=[])
         ast.copy_location(n, node)
         ast.fix_missing_locations(n)
         return n
@@ -327,15 +353,13 @@ class TemplateTransformer(ast.NodeTransformer):
         if not html:
             return node
         n = ast.Name(id='_q_format', ctx=ast.Load())
-        conversion = ast.copy_location(ast.Num(node.conversion), node)
+        conversion = ast.copy_location(_ast_num(node.conversion), node)
         args = [node.value]
         if node.format_spec is not None:
             args += [conversion, node.format_spec]
         elif node.conversion != -1:
             args += [conversion]
-        n = ast.Call(
-            func=n, args=args, keywords=[], starargs=None, kwargs=None
-        )
+        n = ast.Call(func=n, args=args, keywords=[])
         ast.copy_location(n, node)
         ast.fix_missing_locations(n)
         return n
@@ -384,7 +408,7 @@ def main():
             buf = fp.read()
         tree = parse(buf, fn)
         if args.ast:
-            pprint(tree)
+            print(pprint(tree))
         if args.dis:
             co = compile(tree, fn, 'exec')
             dis.dis(co)
